package de.hdm_stuttgart.mi.sd1.filepath;

import java.util.StringTokenizer;

/**
 * Represent file path data.
 *
 */
public class FileMetaInfo {
   
   
   static public final char fileSeparator = System.getProperty("file.separator").charAt(0);
   
   public static final char noDrive = 0;
   
   static final boolean isWindowsOs = System.getProperty("os.name").startsWith("Windows"); // true for Windows 7 / 8 / ...
   
   
   // Drive letter like 'C:' on windows. Value will be set to
   // class variable noDrive's value
   // for relative path specifications or
   // on non-windows platforms.
   public final char drive;
      
   public final String path, basename, extension;

   // Paths may be either relative or absolute.
   public final boolean isRelative;
   
   /**
    * Starting from a string representation of a filename.
    * 
    * @param filePathName e.g. "C:\programms\openvpn\bin\openvpn.exe"
    */
   public FileMetaInfo (final String filePathName) {
      
      final StringTokenizer pathComponents;
      
      if (isWindowsOs) {
         if (filePathName.charAt(1) == ':') { // Absolute path starting with a drive letter like e.g. 'C:'
            isRelative = false;
            drive = filePathName.charAt(0);
            pathComponents = new StringTokenizer(filePathName.substring(2), String.valueOf(fileSeparator)); // Ignoring first two drive related letters
         } else { // relative path not starting with a drive letter
            isRelative = true;
            drive = noDrive;
            pathComponents = new StringTokenizer(filePathName, String.valueOf(fileSeparator));
         }
      } else {
         drive = noDrive;
         if (filePathName.charAt(0) == fileSeparator) {
            isRelative = false;
         } else {
            isRelative = true;
         }
         pathComponents = new StringTokenizer(filePathName, String.valueOf(fileSeparator));
      }
      // Now we must scan for directory separators (either '/' on UNIX or '\' on Windows)

      final StringBuffer directoryComponents;      
      if (isRelative) {
         directoryComponents = new StringBuffer();
      } else {
         directoryComponents = new StringBuffer(String.valueOf(fileSeparator)); // Starting with '/' or '\'
      }
      
      String currentToken;
      do {
         currentToken = pathComponents.nextToken();// May be either a directory or a filename.
         if (pathComponents.hasMoreElements()) { // There is more to come, curentToken still is a directory
            directoryComponents.append(currentToken + fileSeparator);
         }
      } while (pathComponents.hasMoreElements());
      
      path = directoryComponents.
              substring(0, directoryComponents.length() - 1) // Exclude trailing '/' or '\'
              .toString();
      
      final String filename = currentToken;
         
      // Now (if possible) decompose filename xyz.ext into its basename "xyz" and extension "ext"
      
      final int indexOfPointCharacter = filename.indexOf('.');
      
      if (-1 == indexOfPointCharacter) { // Filename does not contain any '.' character
         basename = filename;
         extension = null;
      } else {
         basename = filename.substring(0, indexOfPointCharacter);
         extension = filename.substring(indexOfPointCharacter + 1);
      }
   }

   @Override
   public String toString() {
      
      final StringBuffer filename = new StringBuffer();
      if (isWindowsOs && !isRelative) {
         filename.append(drive);
      }
      return filename.append(path).append(fileSeparator).append(basename).append('.').append(extension).toString();
   }
   
   
}
